@@ -9,7 +9,8 @@ test_that("get_parameterValues()",{
                         globalState= c("M", "U", "M"))
   output <- get_parameterValues()
   output$alpha_pI <- 0.5
-  wrongRootData <- simulate_initialData(infoStr = infoStr, params = output)
+  # Set root data to be the list output of simulate_initialData
+  m <- capture.output(wrongRootData <- simulate_initialData(infoStr = infoStr, params = output), type = "message")
   expect_error(get_parameterValues(rootData = wrongRootData), info = "fails to throw error when argument is not combiStructureInstance")
   # Test return default parameter values
   output <- get_parameterValues()
@@ -18,7 +19,7 @@ test_that("get_parameterValues()",{
   infoStr <- data.frame(n = c(100, 100, 100),
                         globalState= c("M", "U", "M"))
   output$alpha_pI <- 0.5
-  rootData <- simulate_initialData(infoStr = infoStr, params = output)$data
+  m <- capture.output(rootData <- simulate_initialData(infoStr = infoStr, params = output)$data, type = "message")
   output <- get_parameterValues(rootData = rootData)
   expect_equal(output$alpha_pI, 0.5, info ="fails to return rootData parameters when given")
 })
@@ -58,7 +59,8 @@ test_that("simulate_initialData",{
   ## a) For input without customized eqFreqs or params
   infoStr <- data.frame(n = c(100, 100, 100),
                         globalState= c("M", "U", "M"))
-  output <- simulate_initialData(infoStr = infoStr)
+  m <- capture.output(output <- simulate_initialData(infoStr = infoStr), type = "message")
+  expect_equal(m, "Simulating initial data with default parameter values")
   expect_equal(class(output$data)[1], "combiStructureGenerator", info ="does not generate correct output$data class for correct input")
   expect_true(all(output$params == get_parameterValues()), info ="does not return default parameter values when not given a)")
   ## b) For input with customized eqFreqs
@@ -67,7 +69,8 @@ test_that("simulate_initialData",{
                         u_eqFreq = c(0.1, 0.8, 0.1),
                         p_eqFreq = c(0.1, 0.1, 0.1),
                         m_eqFreq = c(0.8, 0.1, 0.8))
-  output <- simulate_initialData(infoStr = infoStr)
+  m <- capture.output(output <- simulate_initialData(infoStr = infoStr), type = "message")
+  expect_equal(m, "Simulating initial data with default parameter values")
   expect_equal(class(output$data)[1], "combiStructureGenerator", info ="does not generate correct output$data class for correct input")
   expect_true(all(output$params == get_parameterValues()), info ="does not return default parameter values when not given b)")
   ## c) For input with customized params
@@ -78,9 +81,26 @@ test_that("simulate_initialData",{
                         m_eqFreq = c(0.8, 0.1, 0.8))
   custom_params <- get_parameterValues()
   custom_params$iota <- 0.5
-  output <- simulate_initialData(infoStr = infoStr, params = custom_params)
+  m <- capture.output(output <- simulate_initialData(infoStr = infoStr, params = custom_params), type = "message")
+  expect_equal(m, "Simulating initial data with customized parameter values")
   expect_equal(class(output$data)[1], "combiStructureGenerator", info ="does not generate correct output$data class for correct input")
   expect_true(all(output$params == custom_params), info ="does not return customized params when given")
+  ## d) For CFTP = TRUE
+  infoStr <- data.frame(n = c(30, 10, 5),
+                        globalState= c("M", "U", "M"))
+  message <- capture.output(o <- simulate_initialData(infoStr = infoStr, CFTP = TRUE), type = "message")
+  expect_equal(message[2], "Calling CFTP algorithm.")
+  # Reset combi counter to check ID of output 
+  # First without CFTP
+  c <- combiStructureGenerator$new(infoStr = infoStr); c$reset_sharedCounter()
+  m <- capture.output(o <- simulate_initialData(infoStr = infoStr), type = "message")
+  expect_equal(o$data$get_id(), 1,
+               info = "wrong ID of generated data without CFTP")
+  # Now with CFTP
+  c <- combiStructureGenerator$new(infoStr = infoStr); c$reset_sharedCounter()
+  m <- capture.output(o <- simulate_initialData(infoStr = infoStr, CFTP = TRUE), type = "message")
+  expect_true(o$data$get_id() == 1,
+               info = "wrong ID of generated data with CFTP")
 })
 
 test_that("simulate_evolData input control",{
@@ -89,8 +109,10 @@ test_that("simulate_evolData input control",{
   infoStr <- data.frame(n = c(100, 100, 100),
                         globalState= c("M", "U", "M"))
   tree <- "(a:1, c:2, (d:3.7, e:4):5);"
-  expect_error(capture.output(simulate_evolData(infoStr = infoStr), type = "message"), info='fails to throw error when tree argument is missing')
-  expect_error(capture.output(simulate_evolData(tree = tree), type = "message"), info='fails to throw error when neither infoStr nor rootData are given')
+  expect_error(capture.output(simulate_evolData(infoStr = infoStr), type = "message"), 
+               info='fails to throw error when tree argument is missing')
+  expect_error(capture.output(simulate_evolData(tree = tree), type = "message"), 
+               info='fails to throw error when neither infoStr nor rootData are given')
 
   ## b) infoStr with eqFreqs: incorrect values
   infoStr <- data.frame(n = c(100, 100, 100),
@@ -98,13 +120,15 @@ test_that("simulate_evolData input control",{
                         u_eqFreq = c(0.1, 0.8, 0.1),
                         p_eqFreq = c(NA, 0.1, 0.1),
                         m_eqFreq = c(0.8, 0.1, 0.8))
-  expect_error(capture.output(simulate_evolData(infoStr = infoStr, tree = tree), type = "message"), info = "fails to throw error when given eqFreqs in infoStr have missing values")
+  expect_error(capture.output(simulate_evolData(infoStr = infoStr, tree = tree), type = "message"), 
+               info = "fails to throw error when given eqFreqs in infoStr have missing values")
   infoStr <- data.frame(n = c(100, 100, 100),
                         globalState = c("M", "U", "M"),
                         u_eqFreq = c(0.1, 0.8, 0.1),
                         p_eqFreq = c(0, 0.1, 0.1),
                         m_eqFreq = c(0.8, 0.1, 0.8))
-  expect_error(capture.output(simulate_evolData(infoStr = infoStr, tree = tree), type ="message"), info = "fails to throw error when given eqFreqs in infoStr are incorrect")
+  expect_error(capture.output(simulate_evolData(infoStr = infoStr, tree = tree), type ="message"), 
+               info = "fails to throw error when given eqFreqs in infoStr are incorrect")
 
   ## c) incorrect customized params
   infoStr <- data.frame(n = c(100, 100, 100),
@@ -120,7 +144,7 @@ test_that("simulate_evolData input control",{
   params <- get_parameterValues()
   infoStr <- data.frame(n = c(100, 100, 100),
                         globalState= c("M", "U", "M"))
-  rootData <- simulate_initialData(infoStr = infoStr)$data
+  m <- capture.output(rootData <- simulate_initialData(infoStr = infoStr)$data, type = "message")
   expect_error(simulate_evolData(rootData = rootData, tree = tree, params = params), info = "fails to throw error when rootData is given and params not NULL")
 
 })
@@ -140,10 +164,10 @@ test_that("simulate_evolData output",{
   expect_equal(output$tree, tree, info ="returns different tree from given one (a1)")
 
   ## a.2) For input without customized eqFreqs or params: input rootData
-  rootData <- simulate_initialData(infoStr = infoStr)$data
+  m <- capture.output(rootData <- simulate_initialData(infoStr = infoStr)$data, type = "message")
   silence <- capture.output(output <- simulate_evolData(rootData = rootData, tree = tree), type = "message")
   expect_equal(class(output$data), "list", info ="does not generate correct output$data list (a2)")
-  print <- sub("\\[\\d+\\] \"(.+):.*\"$", "\\1", silence[2])
+  print <- sub("\\[\\d+\\] \"(.+):.*\"$", "\\1", silence[1])
   expect_print <- "Simulating evolution of given data at root along given tree:  (a:1, c:2, (d:3.7, e:4):5);"
   expect_equal(print, expect_print, info = "Not initiating evolution of given data when rootData is given")
   expect_true(all(output$params == get_parameterValues()), info = "fails to return default params when not given")
@@ -184,14 +208,19 @@ test_that("simulate_evolData output",{
   custom_params$alpha_mNI <- 0.5
   infoStr <- data.frame(n = c(100, 100, 100),
                         globalState= c("M", "U", "M"))
-  rootData <- simulate_initialData(infoStr = infoStr, params = custom_params)$data
+  m <- capture.output(rootData <- simulate_initialData(infoStr = infoStr, params = custom_params)$data, type = "message")
   silence <- capture.output(output <- simulate_evolData(rootData = rootData, tree = tree), type = "message")
   expect_equal(class(output$data), "list", info ="does not generate correct output$data list (d)")
   print <- sub("\\[\\d+\\] \"(.*)\"", "\\1", silence[1])
-  expect_print <- "Parameter values set as in given rootData"
+  expect_print <- "Simulating evolution of given data at root along given tree:  (a:1, c:2, (d:3.7, e:4):5);"
   expect_equal(print, expect_print, info = "Fails to inform that parameter values are set as in given rootData d)")
   expect_equal(output$params$alpha_mNI, 0.5, info = "does not return params as in given rootData")
-
+  
+  ## e) For CFTP
+  infoStr <- data.frame(n = c(10, 10, 20),
+                        globalState= c("M", "U", "M"))
+  silence <- capture.output(output <- simulate_evolData(infoStr = infoStr, tree = tree, CFTP = TRUE), type = "message")
+  expect_equal(silence[3], "Calling CFTP algorithm for data at root before letting it evolve along given tree.")
 })
 
 test_that("simulate_evolData output$data",{
@@ -256,6 +285,41 @@ test_that("simulate_evolData output$data",{
   expect_null(output$data$branchInTree[[6]]$offspring_index)
 })
 
+test_that("simulate_evolData CFTP_step_limit", {
+  
+  tree <- "(a:1,b:1);"
+  infoStr <- data.frame(n = c(5000, 5000, 100),
+                        globalState = c("M", "U", "M"))
+  # Given a number of steps smaller than the number of sites, 
+  # there is no possiblity of convergence in a single iteration
+  # Set a step limit that is applied before the second iteration
+  message <- capture.output(
+    evolData <- simulate_evolData(infoStr,
+                                  tree = tree,
+                                  CFTP = TRUE,
+                                  CFTP_step_limit = 9000),
+    type="message")
+  expect_equal(message[4],
+               "Steps limit reached. Applying approximation for CFTP.")
+})
+
+test_that("simulate_initialData CFTP_step_limit", {
+  
+  tree <- "(a:1,b:1);"
+  infoStr <- data.frame(n = c(5000, 5000, 100),
+                        globalState = c("M", "U", "M"))
+  # Given a number of steps smaller than the number of sites, 
+  # there is no possiblity of convergence in a single iteration
+  # Set a step limit that is applied before the second iteration
+  message <- capture.output(
+    initialData <- simulate_initialData(infoStr,
+                                     CFTP = TRUE,
+                                     CFTP_step_limit = 9000),
+    type="message")
+  expect_equal(message[3],
+               "Steps limit reached. Applying approximation for CFTP.")
+})
+
 
 
 
